/* 
   --------------------------------------------------------------------------------------- 
   File        : A_menu.pde
   Author      : IK5PWC, Civinini Luca - luca@ik5pwc.it - http://www.ik5pwc.it
   Created     : 2011/09/20 
   Description : This file contais all functions for menu management
     
   Licensed freely only to authorized amateur radio stations. Any other use is forbidden.
   Use this code at your own risk.
   Code can be modified and redistributed under following  conditions:
   1) You should not remove any reference to original author
   2) Your changes must be forwared to original  author
   3) Any commercial activity is forbidden
   
   CI-V and Icom are registered trademarks
   --------------------------------------------------------------------------------------- 
*/



#define MENUTIMEOUT  60      // Timeout in seconds for any menu option (i.e. after TIMEOUT without any change, go back to previous menu)
#define NUMCONFIGITEMS 7     // How many main menu items are present?

/* ------------------------------- GLOBAL VARIABLES DECLATATION ---------------------------- */
                                 /*                                                                MENU INDEX                                                                      */
                                /*<------0-------><------1-------><------2-------><------3-------><------4-------><------5-------><------6-------><------7------->*/
const char   menuConfigItems[] = "Switch Config   CI-V Baud Rate  CI-V Rtx AddressCI-V IC-735 modeBand<>Ant configAntenna Names   Software versionFactory defaults";

static unsigned long lastMenuOp = 0;           // This variable stores the "timestamp" for last menu operation
static boolean configChanged = false;          // This is used to check for changes within the menu and trigger EEPROM writes


/*
----------------------------------------------------------------------------------------------------------------------
Sub: configMainMenu
-------------------------------
This function displays the first level menu and allows navigation within it
 

Invoked by:
 . loop

Called Sub/Functions:  
 . lcdConfigMenu()
 . readKey()
 . configBaudRate()
 . configRtxAddress()
 . configIC735Mode()
 . configBandToAntenna()
 . configAntennaName()
 . configSoftwareVersion()
 . configFactoryDefaults()
 . saveConfig()
                
Global Const Used: None 

Global variables used: 
 . configChanged
 . lastMenuOp

Pre Processor Macro: 
 . MENUTIMEOUT
 . NUMCONFIGITEMS
 
Arguments: NONE
*/

void configMainMenu() {
  boolean exitConfig = false;                     // Do we have to exit?
  static byte curSelectedOption = 1;              // Pointer to the option menu item
  
  // Set timestamp when entering into this menu
  lastMenuOp = millis();

  // Set no changes
  configChanged = false;
  
  // Display the menu
  lcdConfigMenu (curSelectedOption,true);

  // We remain in this cycle until timeout or esc
  do {
      switch (readKey()) {
          case 1:      //UP
            //move to next item
            curSelectedOption += 1;

            // if we are at a value less than first menu option, jump to last one         
            if (curSelectedOption == NUMCONFIGITEMS+1) {curSelectedOption = 1;}
            
            // update LCD
            lcdConfigMenu (curSelectedOption,false);

            // update timeout
            lastMenuOp = millis();
            break;

          case 2:      //DOWN
            //move to previous item
            curSelectedOption -= 1;

            // if we are at a value less than first menu option, jump to last one         
            if (curSelectedOption == 0) {curSelectedOption = NUMCONFIGITEMS;}
            
            // Update LCD
            lcdConfigMenu (curSelectedOption,false);

            // update timeout
            lastMenuOp = millis();
            break;
            
          case 4:      //ESC
            // Set exit bit
            exitConfig = true;
            
            // Reset the pointer to first menu
            curSelectedOption = 1;
            break;
            
          case 8:      //SAVE
            // Update timestamp
            lastMenuOp = millis();

            // go to currently selected menu
            switch (curSelectedOption) {
                case 1: configBaudRate(); break;
                case 2: configRtxAddress();break; 
                case 3: configIC735Mode();break; 
                case 4: configBandToAntenna(); break;
                case 5: configAntennaName();break;
                case 6: configSoftwareVersion();break;
                case 7: configFactoryDefaults();break;
            }  
          
            // Display current menu option redesigning the whole display (we're exiting from a submenu)
            lcdConfigMenu (curSelectedOption,true);
            break;
      }
            
  // Exit only whn pressed the button or you timed out 
  } while ((!exitConfig) && (millis() - lastMenuOp <MENUTIMEOUT*1E3)) ;

  // if config was changed somewere, then save in EEPROM
  if (configChanged) {saveConfig();}
  
  // Reset the menu
  curSelectedOption = 1;  
}



/*
----------------------------------------------------------------------------------------------------------------------
Sub: configBaudRate
-------------------------------
This function manages the Baud rate sub menu

Invoked by:
 . configMainMenu

Called Sub/Functions:  
 . lcdConfigBaudRate()
 . lcdPrintAction()
 . readKey()
                
Global Const Used: 
 . baudRates[]

Global variables used: 
 . baudRateIndex
 . lastMenuOp
 . configChanged

Pre Processor Macro: 
 . MENUTIMEOUT
 
Arguments: NONE
*/

void configBaudRate() {
  boolean exitConfig = false;                       // Do we have to exit?
  byte curSelectedOption = 0;                        // Stores currently selected value

  // By default get the current value
  curSelectedOption= baudRateIndex;
  
  // LCD DISPLAY
  lcdConfigBaudRate(curSelectedOption,true);

  // We remain in this cicle until exit
  do {
      switch (readKey()) {
          case 1:      //UP
            //move to previous item
            curSelectedOption += 1;

            // if we are at a value less than first menu option, jump to last one         
            if (curSelectedOption > sizeof(baudRates)/sizeof(int)-1) {curSelectedOption = 1;}
            
            // Update LCD 
            lcdConfigBaudRate(curSelectedOption,false);

            // update timeout
            lastMenuOp = millis();
            break;

          case 2:      //DOWN
            //move to previous item
            curSelectedOption -= 1;

            // if we are at a value less than first menu option, jump to last one         
            if (curSelectedOption == 0) {curSelectedOption = sizeof(baudRates)/sizeof(int)-1;}

            // Update LCD 
            lcdConfigBaudRate(curSelectedOption,false);

            // update timeout
            lastMenuOp = millis();
            break;

          case 4:      //ESC
            // Set exit flag
            exitConfig = true;

            // Print action
            lcdPrintAction(false);

            // update timeout
            lastMenuOp = millis();
            break;            

          case 8:      //SAVE
            // Save the value
            baudRateIndex = curSelectedOption;
                        
            // Print the action
            lcdPrintAction(true);
            
            // Set config changed flag
            configChanged = true;
            
            // Close and open serial port with new values
            Serial.end();
            Serial.begin (baudRates[baudRateIndex]);

            // Set exit flag
            exitConfig = true;            

            // update timeout
            lastMenuOp = millis();
            break;
      }
            
  // Exit only whn pressed the button or you timed out 
  } while ((!exitConfig) && (millis() - lastMenuOp <MENUTIMEOUT*1E3)) ; 
}



/*
----------------------------------------------------------------------------------------------------------------------
Sub: configRtxAddress
-------------------------------
This function manages the CI-V address setting

Invoked by:
 . configMainMenu

Called Sub/Functions:  
 . lcdConfigRtxAddress()
 . readKey()
 . lcdPrintAction()
                
Global Const Used: NONE

Global variables used: 
 . rtxAddr
 . lastMenuOp
 . configChanged

Pre Processor Macro: 
 . MENUTIMEOUT
 
Arguments: NONE
*/

void configRtxAddress() {
  boolean exitConfig = false;                              // Do we have to exit?
  byte   newRtxAddress  ;                                 // Stores the value within the function

  // By default get the current value
  newRtxAddress = rtxAddr;
  
  // LCD DISPLAY
  lcdConfigRtxAddress(newRtxAddress,true);
  
  // We remain in this cicle until exit
  do {
      switch (readKey()) {
          case 1:      //UP
            newRtxAddress += 1;
            
            // Last valid value is 0xDF, so 0xE0 should be redefined to first valid value
            if (newRtxAddress == 0xE0) {newRtxAddress = 0x01;}
            
            //Update LCD
            lcdConfigRtxAddress(newRtxAddress,false);

            // update timeout
            lastMenuOp = millis();
            break;

          case 2:      //DOWN
            // Change value
            newRtxAddress -= 1;
            
            //Update LCD
            lcdConfigRtxAddress(newRtxAddress,false);

            // First valid value is 0x01, so 0x00 should be redefined to last valid value
            if (newRtxAddress == 0x00) {newRtxAddress = 0xDF;}

            // update timeout
            lastMenuOp = millis();
            break;

          case 4:      //ESC
            // Set exit flag
            exitConfig = true;

            // Print action
            lcdPrintAction(false);           

            // update timeout
            lastMenuOp = millis();
            break;            

          case 8:      //SAVE
            // Save the value
            rtxAddr = newRtxAddress;
            
            // Set config changed flag
            configChanged = true;

            // Print the action
            lcdPrintAction(true);

            // update timeout
            lastMenuOp = millis();

            // Set exit flag
            exitConfig = true;            
            break;
      }
      
  // Exit only whn pressed the button or you timed out 
  } while ((!exitConfig) && (millis() - lastMenuOp <MENUTIMEOUT*1E3)) ;    
}


/*----------------------------------------------------------------------------------------------------------------------
Sub: configIC735Mode
-------------------------------
This function manages the IC735 mode option

Invoked by:
 . configMainMenu

Called Sub/Functions:  
 . lcdConfigIC735Mode
 . readKey
 . lcdPrintAction
                
Global Const Used: NONE

Global variables used: 
 . ic735mode
 . lastMenuOp
 . configChanged

Pre Processor Macro: 
 . MENUTIMEOUT
 
Arguments: NONE
*/

void configIC735Mode() {
  boolean exitConfig = false;                              // Do we have to exit?
  boolean isIC735    = false;                             // Stores currently selected value

  // By default get the current value
  isIC735= ic735mode;
  
  // LCD DISPLAY
  lcdConfigIC735Mode(isIC735,true);
  
  // We remain in this cicle until exit
  do {
      switch (readKey()) {
          case 1:      //UP
            // Change value
            isIC735 = !isIC735;

            // LCD Update
            lcdConfigIC735Mode(isIC735,false);
  
            // update timeout
            lastMenuOp = millis();
            break;

          case 2:      //DOWN 
            // Change value
            isIC735 = !isIC735;

            // LCD Update
            lcdConfigIC735Mode(isIC735,false);

            // update timeout
            lastMenuOp = millis();
            break;

          case 4:       //EXIT 
            // Set exit flag
            exitConfig = true;

            // Print action
            lcdPrintAction(false);

            // update timeout
            lastMenuOp = millis();
            break;            

          case 8:      //SAVE
            // Save the value
            ic735mode = isIC735;
            
            // Set config changed flag
            configChanged = true;

            // Print the action
            lcdPrintAction(true);

            // update timeout
            lastMenuOp = millis();

            // Set exit flag
            exitConfig = true;            
            break;
      }
      
  // Exit only whn pressed the button or you timed out 
  } while ((!exitConfig) && (millis() - lastMenuOp <MENUTIMEOUT*1E3)) ;    
}




/*----------------------------------------------------------------------------------------------------------------------
Sub: configBandToAntenna
-------------------------------
This sub associates an antenna to a band

Invoked by:
 . configMainMenu

Called Sub/Functions:  
 . lcdConfigBandToAntenna
 . readKey
 . lcdPrintAction
                
Global Const Used: NONE

Global variables used: 
 . bandToAnt[]
 . lastMenuOp
 . configChanged

Pre Processor Macro: 
 . NUMANTENNA
 . NUMBANDS
 . MENUTIMEOUT
 
Arguments: NONE
*/

void configBandToAntenna() {
  boolean exitConfig = false;                              // Do we have to exit?
  byte    antennaIndex=0 ;                                 // Antenna Index when cycling antennas (start value doesn't matter)
  byte    bandIndex = 1;                                   // Band index when cycling bands (start from first band)
  boolean scrollBand = true;                               // true - scrolling bands : false - scrolling antenna within a band
  
  // LCD DISPLAY
  lcdConfigBandToAntenna(bandIndex,antennaIndex,scrollBand,true);
  
  // We remain in this cicle until exit
  do {      
      // Get the key
      switch (readKey()) {
          case 1:        // UP
            // Check to see if i'm scrolling the bands
            if (scrollBand) {
              // Yes, increment the band
              bandIndex += 1;
              
              // If at the end, move to first band
              if (bandIndex == NUMBANDS+1) {bandIndex = 1;}
            } else {
              // no, I'm scrolling antennas
              antennaIndex +=1;
              
              // Last antenna?
              if (antennaIndex == NUMANTENNA + 1) {antennaIndex = 0;}
            }  

            // LCD Update
            lcdConfigBandToAntenna(bandIndex,antennaIndex,scrollBand,false);
            
            // update timeout
            lastMenuOp = millis();
            break;

          case 2:        // DOWN
            // Check to see if i'm scrolling the bands
            if (scrollBand) {
                // Yes, decrement the band            
                bandIndex -= 1;
            
                // If at the end, move to first band
                if (bandIndex == 0) {bandIndex = NUMBANDS;}
            } else {
                // no, I'm scrolling antennas
                antennaIndex -=1;
              
                // first antenna antenna? (antennaIndex is a byte so -1 equals 0xFF)
                if (antennaIndex == 0xFF) {antennaIndex = NUMANTENNA;}
            }

            // LCD Update
            lcdConfigBandToAntenna(bandIndex,antennaIndex,scrollBand,false);

            // update timeout
            lastMenuOp = millis();
            break;

          case 4:        // ESC
            if (scrollBand) {
                // We're scrolling  bands,  so exiting to previous menu
                exitConfig = true;
            } else {
                // We are assigning antennas, cancel operation and back to band scrolling
                lcdPrintAction(false);
                scrollBand = true;
                lcdConfigBandToAntenna(bandIndex,antennaIndex,scrollBand,true);
            }

            // update timeout
            lastMenuOp = millis();
            break;            

          case 8:         // SAVE
            if (scrollBand) {
               // move to scrolling antenna
               scrollBand = false;

               // get the antenna value for specific band    
               antennaIndex = bandToAnt[bandIndex];
   
               // LCD Update
               lcdConfigBandToAntenna(bandIndex,antennaIndex,scrollBand,false);

            } else {
               // Save the antenna
               bandToAnt[bandIndex]=antennaIndex;
               
               // Set config changed flag
               configChanged = true;
               
               // Print the action
               lcdPrintAction(true);             
               scrollBand = true;
               
               // Update LCD
               lcdConfigBandToAntenna(bandIndex,0,scrollBand,true);
            }

            // update timeout
            lastMenuOp = millis();
            break;
      }
      
  // Exit only whn pressed the button or you timed out 
  } while ((!exitConfig) && (millis() - lastMenuOp <MENUTIMEOUT*1E3)) ;    
}




/*----------------------------------------------------------------------------------------------------------------------
Sub: configAntennaName       
-------------------------------
This sub manages antenna names

Invoked by:
 . configMainMenu

Called Sub/Functions:  
 . lcdConfigAntennaName
 . readKey
 . lcdPrintAction
 . getValidCharIndex
                
Global Const Used: 
 . validChars[]
 . antennaName[]

Global variables used: 
 . lastMenuOp
 . configChanged
 
Pre Processor Macro: 
 . ANTENNANAMELEN
 . NUMANTENNA
 . MENUTIMEOUT
 
Arguments: NONE
*/

void configAntennaName() {
  boolean exitConfig = false;                              // Do we have to exit?
  byte    antennaIndex=1 ;                                 // Antenna Index when cycling antennas 
  boolean scrollAntenna = true;                            // true - scrolling antennas : false - scrolling antenna name
  byte    indexChar = 0;                                   // point to current char position
  byte    indexValidChar = 0;                              // Store the ascii value for a char (starts with A)
  char    tempName[ANTENNANAMELEN];                        // store the name while editing
  
  // LCD DISPLAY
  lcdConfigAntennaName(antennaIndex, scrollAntenna, tempName, indexChar,true);

  // We remain in this cicle until exit
  do {      
      // Get the key
      switch (readKey()) {
          case 1:        // UP
            if (scrollAntenna) {
                // Scrolling antennas
                antennaIndex +=1;
              
                // Last antenna?
                if (antennaIndex == NUMANTENNA + 1) {antennaIndex = 1;}
            } else {
                // move to next char
                indexValidChar +=1;
                
                // check if it's the last one
                if (indexValidChar == sizeof(validChars)-1) {indexValidChar = 0;}
                
                // save new value in the temp array
                tempName[indexChar]  = validChars[indexValidChar];
            }

            // LCD DISPLAY
            lcdConfigAntennaName(antennaIndex, scrollAntenna, tempName, indexChar,false);
            
            // update timeout
            lastMenuOp = millis();
            break;

          case 2:        // DOWN
            // Check to see if i'm scrolling the antenna
            if (scrollAntenna) {
                // Yes, decrement the antenna            
                antennaIndex -= 1;
            
                // first antenna antenna?
                if (antennaIndex == 0) {antennaIndex = NUMANTENNA;}
            } else {
                // no, I'm scrolling chars
                indexValidChar -=1;
                
                // check if it's the last one (indexValidChar is a byte so -1 equals 0xFF)
                if (indexValidChar == 0xFF) {indexValidChar = sizeof(validChars)-2;}
                
                // save innew value in the temp array
                tempName[indexChar]  = validChars[indexValidChar];
            }

            // LCD DISPLAY
            lcdConfigAntennaName(antennaIndex, scrollAntenna, tempName, indexChar,false);
            
            // update timeout
            lastMenuOp = millis();
            break;

          case 4:        // ESC
            if (scrollAntenna) {
                // We're scrolling  antennas,  so exiting to previous menu
                exitConfig = true;
            } else {
               // we are scrolling names.. Save the current value and move to previous char
               tempName[indexChar] = validChars[indexValidChar];
               indexChar -= 1;

               // Was the first char? (indexChar is a byte so -1 equals 0xFF)
               if (indexChar == 0xFF) {
                    // Exiting to scroll antenna names
                    
                   // Print the action
                   lcdPrintAction(false);             
                   
                   // back to antenna scrolling
                   scrollAntenna = true;
                   
                   // Update LCD so that it reflect antenna name config (old one, we ESCaped to first char)
                   lcdConfigAntennaName(antennaIndex, scrollAntenna, tempName, indexChar,true);               
               }  else {
                   // We are not at the beginning of the string, so get the current char 
                  indexValidChar = getValidCharIndex(tempName[indexChar]);

                  // LCD DISPLAY
                  lcdConfigAntennaName(antennaIndex, scrollAntenna, tempName, indexChar,false);
               }  
            }

            // update timeout
            lastMenuOp = millis();
            break;            

          case 8:         // SAVE
            if (scrollAntenna) {
               // move to scrolling antenna names
               scrollAntenna = false;
              
               // init the temp name
               for (int i=0; i<ANTENNANAMELEN; i++) {tempName[i] = antennaName[antennaIndex*ANTENNANAMELEN + i];}

               // start from first antenna name char
               indexChar = 0;
               indexValidChar = getValidCharIndex(tempName[indexChar]);
 
               // Update LCD 
               lcdConfigAntennaName(antennaIndex, scrollAntenna, tempName, indexChar,false);               


            } else {
               // we are scrolling names.. Save the current value and move to next char
               tempName[indexChar] = validChars[indexValidChar];
               indexChar += 1;
               
               // Was the last char?
               if (indexChar == ANTENNANAMELEN) {
                   // yes, save and exit
                   for (int i=0; i<ANTENNANAMELEN; i++) {antennaName[antennaIndex*ANTENNANAMELEN + i] = tempName[i];}

                   // Print the action
                   lcdPrintAction(true);             
                   
                   // Set config changed flag
                   configChanged = true;

                   // back to antenna scrolling
                   scrollAntenna = true;
                   
                   // Update LCD so that it reflect antenna name config (new value)
                   lcdConfigAntennaName(antennaIndex, scrollAntenna, tempName, indexChar,false);               
               } else {
                   // No get the next char 
                   indexValidChar = getValidCharIndex(tempName[indexChar]);
                   
                   // LCD DISPLAY
                   lcdConfigAntennaName(antennaIndex, scrollAntenna, tempName, indexChar,false);
               }  
            }

            // update timeout
            lastMenuOp = millis();
            break;
      }
      
  // Exit only whn pressed the button or you timed out 
  } while ((!exitConfig) && (millis() - lastMenuOp <MENUTIMEOUT*1E3)) ;    
}



/*----------------------------------------------------------------------------------------------------------------------
 Sub: configFactoryDefaults
-------------------------------
This function restores factory settings

Invoked by:
 . configMainMenu()

Called Sub/Functions:  
 . lcdConfigFactoryDefaults()
 . readKey()
 . lcdPrintAction()
 . initDefaults()
 
Global Const Used: 
 . lastMenuOp
 . configChanged

Global variables used: NONE

Pre Processor Macro: 
 . MENUTIMEOUT
 
Arguments: NONE
*/
void configFactoryDefaults() {
  boolean exitConfig = false;                              // Do we have to exit?
 
  // LCD DISPLAY
  lcdConfigFactoryDefaults();
  
  // We remain in this cicle until exit
  do {
      switch (readKey()) {
          case 4:       //EXIT 
            // Set exit flag
            exitConfig = true;

            // Print action
            lcdPrintAction(false);
            break;            

          case 8:      //SAVE
            initDefaults();

            // Set config changed flag
            configChanged = true;

            // Print the action
            lcdPrintAction(true);

            // Set exit flag
            exitConfig = true;            
            break;
      }
      
  // Exit only whn pressed the button or you timed out 
  } while ((!exitConfig) && (millis() - lastMenuOp <MENUTIMEOUT*1E3)) ;    
}



/*----------------------------------------------------------------------------------------------------------------------
 Sub: configSoftwareVersion
-------------------------------
This function prints the software version

Invoked by:
 . configAntennaName()

Called Sub/Functions:  
 . lcdConfigSoftwareVersion()
 . readKey()
 
Global Const Used: 
 . lastMenuOp

Global variables used: NONE

Pre Processor Macro: 
 . MENUTIMEOUT
 
Arguments: NONE
*/

void configSoftwareVersion() {
  boolean exitConfig = false;                              // Do we have to exit?
  
  // LCD DISPLAY
  lcdConfigSoftwareVersion();
  
  // We remain in this cicle until exit
  do {
      switch (readKey()) {
          case 4:      //ESC
            // Set exit flag
            exitConfig = true;

            // update timeout
            lastMenuOp = millis();
            break;            
      }
      
  // Exit only whn pressed the button or you timed out 
  } while ((!exitConfig) && (millis() - lastMenuOp <MENUTIMEOUT*1E3)) ;    
}



/* --------------------------------------------------------- COMMON MENU FUNCTIONS ---------------------------------------------------------*/



/*----------------------------------------------------------------------------------------------------------------------
Function: getValidCharIndex
-------------------------------
Get the char position within the validChars array

Invoked by:
 . configAntennaName()

Called Sub/Functions:  NONE
                
Global Const Used: 
 . validChars[]

Global variables used: NONE

Pre Processor Macro: NONE
 
Arguments: 
 . value: the char being searched within the validChars array
*/
int getValidCharIndex (char value) {
      
     // By default, return nothing
     int i=0;
     
     // get the proper indexValidChar cycling the validChars array
     for (i ; i < sizeof(validChars); i++) {
       
         // if it's the same char, then exit
         if (validChars[i] == value) {break;}
     } 

    return(i);
}






