/* 
   --------------------------------------------------------------------------------------- 
   File        : B_lcd.pde
   Author      : IK5PWC, Civinini Luca - luca@ik5pwc.it - http://www.ik5pwc.it
   Created     : 2011/09/20 
   Description : This file contais all functions for menu management
   
   Licensed freely only to authorized amateur radio stations. Any other use is forbidden.
   Use this code at your own risk.
   Code can be modified and redistributed under following  conditions:
   1) You should not remove any reference to original author
   2) Your changes must be forwared to original  author
   3) Any commercial activity is forbidden
   
   CI-V and Icom are registered trademarks
   --------------------------------------------------------------------------------------- 
*/

static const int LCDCOLS=16;              // Size for LCD display 
static const int LCDROWS=2;               // Colunns and rows

#define LCDRS 13                          // LCD Definition - R/S PIN
#define LCDEN 12                          // LCD Definition - Enable PIN
#define LCDD4 11                          // LCD Data 4
#define LCDD5 10                          // LCD Data 5
#define LCDD6 9                           // LCD Data 6
#define LCDD7 8                           // LCD Data 7

#define INITTITLEDELAY  3000             // Milliseconds to display the project title
#define INITAUTHORDELAY 1500             // milliseconds to display author
#define ACTIONDELAY     1500             // milliseconds to display action on menu


// Include LCD Library and declare an LCD object (? dunno if it's the correct name)
#include <LiquidCrystal.h>

LiquidCrystal lcd(LCDRS, LCDEN, LCDD4, LCDD5, LCDD6, LCDD7);


/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdInitWithBanner
-------------------------------
 This Sub init the display at program's
 startup.
 
Invoked by:
 . setup

Called Sub/Functions: NONE
                
Global Const Used: 
 . LCDCOLS
 . LCDROWS

Global variables used: NONE
 
Pre Processor Macro: 
 . INITTITLEDELAY
 . VERSION
 
Arguments: NONE
*/

void lcdInitWithBanner() {
  
  // Init the display and clear the content
  lcd.begin(LCDCOLS, LCDROWS);
  lcd.clear();

  // Display Banner - Device name
  lcd.setCursor(0,0);
  lcd.print("CIV based Remote");lcd.setCursor(0,1); lcd.print("Antenna Switch");
  delay (INITTITLEDELAY);
  
  // Display Banner - Version and Author
  lcd.clear();lcd.setCursor(0,0);lcd.print("Version ");lcd.print(VERSION);
  lcd.setCursor(0,1);
  lcd.print("   by  IK5PWC   ");
  delay (INITAUTHORDELAY);

  // Clear
  lcd.clear();
}


/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdUpdate 
-------------------------------
 This sub updates LCD status during normal
 operation. Informations on LCD are updated only
 if they have changed from last update. Is it 
 possible to force update.
 
Invoked by:
 . setup
 . loop

Called Sub/Functions: NONE
                
Global Const Used: 
 . bandMeters[]

Global variables used: 
 . antennaName[]
 
Pre Processor Macro: 
 . ANTENNANAMELEN
 
Arguments: 
 . newFreq: the frequency to be written to display
 . newBand: the band to be written to display
 . newAntenna: the currently selected antenna index  
 . forceUpdate: if true, then display is rewritten also if no change has been done
 
*/

void lcdUpdate(long newFreq, byte newBand, byte newAntenna, boolean forceUpdate) {
  static long         prevFreq    = 0;          // Current  frequency value
  static byte         prevBand    = 0;          // Current and new band value
  static byte         prevAntenna = 0 ;         // Current selected antenna
   
  // if force clear the display
  if (forceUpdate) {lcd.clear();}  

  // Frequency UPDATE
  if ((newFreq != prevFreq) || forceUpdate) {
     // move the cursor to frequency area;
      lcd.setCursor(0,0);
   
     // set spaces in the frequency position (set to blank)
     lcd.print("     ");

     // set the starting point for the frequency based on the value (4 or 5 digits)
     if (newFreq < 1E4)  {lcd.setCursor(1,0);} else {lcd.setCursor(0,0);}

     // Print the frequency
     if (newFreq == 0) {lcd.print(" n/a");} else {lcd.print(newFreq);}
  
     // Move to the frequency unit and print it
     lcd.print(" KHz");

     // Update previous frequency
     prevFreq = newFreq; 
  }
  
  // Band UPDATE
  if ((newBand != prevBand) || forceUpdate){
    // move the cursor to BAND area
    lcd.setCursor(10,0);

    // Check if it's a valid band
    if (newBand == 0) {
      lcd.print("NotHam");  
    } else {
      // Always put two spaces
      lcd.print("  ");   
      
      // determine how long is the string to be printed and print the value in the appropriate position
      if (bandMeters[newBand] >= 100) {lcd.print(bandMeters[newBand],DEC);} 
      if (bandMeters[newBand] >= 10) {lcd.print(" "); lcd.print(bandMeters[newBand],DEC);}
      if (bandMeters[newBand] < 10) {lcd.print("  "); lcd.print(bandMeters[newBand],DEC);} 
      
      // The "m" char will be always on the last col
      lcd.setCursor(15,0);
      lcd.print("m");
    }
    
    // Save the current band value
    prevBand = newBand;
  }    

  // Antenna UPDATE
  if ((newAntenna != prevAntenna) || forceUpdate){
      // Move LCD cursor to proper position
      lcd.setCursor(0,1);
      
      // Print defined items
      lcd.print("Ant "); lcd.print(newAntenna,DEC); lcd.print(": ");
      
      // Print antenna Name
      for (int i=0 ; i<ANTENNANAMELEN ; i++) {lcd.print(antennaName[newAntenna*ANTENNANAMELEN +i]);}

      // Update value
      prevAntenna = newAntenna;
  }     
}



/* ---------------------------------------------------------  MENU MANAGEMENT FUNCTIONS ---------------------------------------------------------*/



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdConfigMenu 
-------------------------------
 This Sub prints on LCD the proper description for 1st level menu. 
 
Invoked by:
 . configMainMenu
 . loop

Called Sub/Functions: 
 . lcdCurrentConfigItem
                
Global Const Used: 
 . menuConfigItems[]

Global variables used: NONE
 
Pre Processor Macro: 
 . LCDCOLS
 
Arguments: 
 . configMenuID: is an index to the menu description stored within the menuConfigItems array
 . clearLCD: redraw entire display (normally only second line is affected)

*/

void lcdConfigMenu(byte configMenuID, boolean clearLcd) {
 
  // If in force mode, print also the first row 
  if (clearLcd) {lcdCurrentConfigItem(0);}
  
  // Clear the second row and print the proper item
  lcdClearRow(1);
  for (int i =0 ; i < LCDCOLS; i++) {lcd.print(menuConfigItems[configMenuID*LCDCOLS + i]); }
      
}



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdConfigBaudRate 
-------------------------------
 This sub manages LCD messaged for the baudrate config option. 
 
Invoked by:
 . configBaudRate

Called Sub/Functions: 
 . lcdCurrentConfigItem
 . lcdClearRow
                
Global Const Used: 
 . baudRates[]

Global variables used: NONE
 
Pre Processor Macro: NONE
 
Arguments: 
 . baudRate: is an index to valid baudrates stored within the baudRates array
 . clearLCD: redraw entire display (normally only second line is affected) 

*/

void lcdConfigBaudRate(byte baudRate, boolean clearLcd) {
 
  // If in force mode, print also the first row 
  if (clearLcd) {lcdCurrentConfigItem(1);}
  
  // Print the baudrate value on second row and set blinking if different from previous value or forced to update
  lcdClearRow(1);      // erase second row

  // Move to the beginning, display baud rate and start blinking the first char
  lcd.print(baudRates[baudRate]);
  lcd.setCursor(0,1);  
  lcd.blink();
}



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdConfigRtxAddress 
-------------------------------
 This Sub manages the CIV Address
 
Invoked by:
 . configRtxAddress

Called Sub/Functions: 
 . lcdCurrentConfigItem
 . lcdClearRow
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: NONE
 
Arguments: 
 . address: is the address to be printed
 . clearLCD: redraw entire display (normally only second line is affected)

*/

void lcdConfigRtxAddress (byte address, boolean clearLcd) {
 
  // If in force mode, print also the first row 
  if (clearLcd) {lcdCurrentConfigItem(2);lcd.noBlink();}
  
  // erase all second row
  lcdClearRow(1);

  // Move to the beginning, display the address in hex format, append "h" and start blinking the first char
  if (address < 0x10) {lcd.print("0");}
  lcd.print(address,HEX);
  lcd.print("h");
  lcd.setCursor(3,1);  
  lcd.blink();

}



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdConfigIC735Mode 
-------------------------------
This sub manages the IC-735 option which in 
turn defines how long should be the frequency format.
 
Invoked by:
 . configIC735Mode

Called Sub/Functions: 
 . lcdCurrentConfigItem
 . lcdClearRow
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: NONE
 
Arguments: 
 . value: true if IC735, otherwise false
 . clearLCD: redraw entire display (normally only second line is affected)

*/

void lcdConfigIC735Mode(boolean value, boolean clearLcd) {
 
  // If in force mode, print also the first row 
  if (clearLcd) {lcdCurrentConfigItem(3);}
  
  // erase all second row
  lcdClearRow(1);

  // Move to the beginning, display baud rate and start blinking the first char
  if (value) { lcd.print("ON");} else {lcd.print("OFF");}
  lcd.setCursor(0,1);  
  lcd.blink();
}



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdConfigBandToAntenna 
-------------------------------
This sub manages the band to antenna configuration
 
Invoked by:
 . configBandToAntenna

Called Sub/Functions: 
 . lcdCurrentConfigItem
 . lcdClearRow
                
Global Const Used: 
 . bandMeters[]

Global variables used: 
 . antennaName[]
 
Pre Processor Macro: 
 . ANTENNANAMELEN
 
Arguments: 
 . bandIndex: currently being configured band
 . bandScrolling: true if scrolling band, false if scrolling antenna for a specific band
 . antennaIndex: an index to the antenna associated to bandIndex
 . clearLCD: redraw entire display (normally only second line is affected)

*/

void lcdConfigBandToAntenna(byte bandIndex, byte antennaIndex, boolean bandScrolling, boolean clearLcd) {
  byte tmpAntennaIndex = 0;                // Temporary antenna Index
  
  // If in force mode, print also the first row 
  if (clearLcd) { lcdCurrentConfigItem(4);}
  
  if (bandScrolling) {   
        // Clear second row
        lcdClearRow(1);
 
        // Update the band indicator
        if (bandMeters[bandIndex]<100) { lcd.print(" ");}      // for band with 2 digit
        if (bandMeters[bandIndex]<10) { lcd.print(" ");}       // Additional space for band with one digit
        lcd.print(bandMeters[bandIndex],DEC);
        lcd.print("m>"); 

        //If we are scrolling bands, then get the antenna index from array 
        tmpAntennaIndex = bandToAnt[bandIndex];

  } else {
        // clear only the antenna part within the second row
        lcd.setCursor(8,1);
        lcd.print("         ");
        
        // The index to be used for printing name is the one passed
        tmpAntennaIndex = antennaIndex;
  }
  
  // Print antenna name
  lcd.setCursor(5,1);
  lcd.print(tmpAntennaIndex,DEC);
  lcd.print (" ");      
  for (int i = 0; i <ANTENNANAMELEN;i++) { lcd.print (antennaName[tmpAntennaIndex*ANTENNANAMELEN + i]);}
  
  // if not in bandscrolling, then blink antenna selection
  if (!bandScrolling) {
      lcd.setCursor(5,1);
      lcd.blink();        
  }  else { lcd.noBlink();}     
}



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdConfigAntennaName 
-------------------------------
This sub manages the antenna name
 
Invoked by:
 . configAntennaName

Called Sub/Functions: 
 . lcdCurrentConfigItem
 . lcdClearRow
                
Global Const Used: NONE

Global variables used: 
 . antennaName[]
 
Pre Processor Macro: 
 . ANTENNANAMELEN
 
Arguments: 
 . antennaIndex is the antenna being displayed/edited
 . antScrolling indicates if we are scrolling antennas or name
 . newName is the new antenna name to be printed (array)
 . charPosition: in the currently edited char within antenna name
 . clearLCD: redraw entire display (normally only second line is affected)

*/

void lcdConfigAntennaName(byte antennaIndex, boolean antScrolling, char newName[], byte charPosition, boolean clearLcd) {
  
  // If in force mode, print also the first row 
  if (clearLcd) { lcdCurrentConfigItem(5);lcd.noBlink();}

  // Second row must be updated only when forced, when antScrolling with different antenna then previous update or when not antScrolling with different char
  if (antScrolling)  {
      // Clear bliking
      lcd.noBlink();
        
      // Clear second row
      lcdClearRow(1);
   
     // Update the Antenna Indicator
     lcd.print("Ant ");
     lcd.print(antennaIndex,DEC);
     lcd.print(": ");
       
     // Print the antenna name
     for (int i = 0; i <ANTENNANAMELEN;i++) { lcd.print (antennaName[antennaIndex*ANTENNANAMELEN + i]);}
  } else {   
       lcd.setCursor(7,1);

       //Print the temporary antenna name
       for (int i = 0; i <ANTENNANAMELEN;i++) { lcd.print (newName[i]);}
       
       // Blink the edited char position
       lcd.setCursor(7+charPosition,1);
       lcd.blink();
  }  
}




/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdConfigSoftwareVersion 
-------------------------------
This sub prints the software version
 
Invoked by:
 . configSoftwareVersion

Called Sub/Functions: 
 . lcdCurrentConfigItem
 . lcdClearRow
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: NONE
 
Arguments: NONE

*/

void lcdConfigSoftwareVersion() {
  // Print first line
  lcdCurrentConfigItem(6);
  lcd.noBlink();
  
  // Display Version 
  lcdClearRow(1);
  lcd.print(VERSION);
  lcd.print(" by IK5PWC");
}



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdConfigFactoryDefaults 
-------------------------------
This sub manages the reset fo factory options
 
Invoked by:
 . configFactoryDefaults

Called Sub/Functions: 
 . lcdCurrentConfigItem
 . lcdClearRow
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: NONE
 
Arguments: NONE

*/

void lcdConfigFactoryDefaults() {
 
  lcdCurrentConfigItem(7);
  lcd.noBlink();
  
  // Clear second row and stamp a message
  lcdClearRow(1);  
  lcd.print("Confirm Reset?");
}



/*----------------------------------------------------------------------------------------------------------------------
Sub: displayMessage 
-------------------------------
This sub displays a message (if any)
 
Invoked by:
 . loop

Called Sub/Functions: 
 . lcdClearRow
 . readKey
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: NONE
 
Arguments: 
 . message: a byte value for a predefined set of messages
    0x01: Frequency is not a valid HAM band
    0x02: No antenna for current band
    0x04: No data received from radio 
    0x08: Manual mode enabled 
    0x10: All antennas are disconnected
*/

void displayMessage(byte message) {
  boolean exitConfig = false;                              // Do we have to exit? by default Yes, IE by default there is no message
  unsigned long timeStamp=0;
  
  // init timestamp
  timeStamp = millis();
  
  // Clear display
  lcdClearRow(1);
  lcdClearRow(0);

  switch (message) {
    case 0x01:
        lcd.print("Frequency is not");
        lcd.setCursor(0,1);
        lcd.print("a valid HAM band");
        break;
   
    case 0x02:
        lcd.print("No antenna for  ");
        lcd.setCursor(0,1);
        lcd.print("current band    ");
        break;
   
    case 0x04:
        lcd.print("No data received");
        lcd.setCursor(0,1);
        lcd.print("from radio      ");
        break;
    
    case 0x08: 
        lcd.print("Manual mode     ");
        lcd.setCursor(0,1);
        lcd.print("enabled         ");
        break;

    case 0x10: 
        lcd.print("All antennas are");
        lcd.setCursor(0,1);
        lcd.print("disconnected    ");
        break;
  }

  do {
      // We remain in this cicle until MSG has been pressed or timeout is reached
      if (readKey() == 16 ) {exitConfig = true;   }
      
  // Exit only whn pressed the button or you timed out 
  } while ((!exitConfig) && (millis() - timeStamp <5*1E3)) ;    
}

/* --------------------------------------------------------- COMMON LCD FUNCTIONS ---------------------------------------------------------*/



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdCurrentConfigItem 
-------------------------------
This sub prints (after clearing) on the first LCD row 
 the config option being edited
 
Invoked by:
 . lcdConfigMenu
 . lcdConfigBaudRate
 . lcdConfigRtxAddress
 . lcdConfigIC735Mode
 . lcdConfigBandToAntenna
 . lcdConfigAntennaName
 . lcdConfigSoftwareVersion
 . lcdConfigFactoryDefaults

Called Sub/Functions: 
 . lcdClearRow
                
Global Const Used: 
 . menuConfigItems[]

Global variables used: NONE
 
Pre Processor Macro: 
 . LCDCOLS
 
Arguments: 
 . menuIndex: a pointer to menuConfigItems array
*/

void lcdCurrentConfigItem(byte menuIndex) {
  // Clear the row
  lcdClearRow(0);
  
  // Print the string
  for (int i =0 ; i < LCDCOLS; i++) {lcd.print(menuConfigItems[menuIndex*LCDCOLS + i]); }  
}



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdClearRow 
-------------------------------
This sub clears a specific LCD row. 
 
Invoked by:
 . lcdConfigMenu
 . lcdConfigBaudRate
 . lcdConfigRtxAddress
 . lcdConfigIC735Mode
 . lcdConfigBandToAntenna
 . lcdConfigAntennaName
 . lcdConfigSoftwareVersion
 . lcdConfigFactoryDefaults
 . displayMessage
 . lcdCurrentConfigItem

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: 
 . LCDCOLS
 
Arguments: 
 . rowID: the row being cleared (0 or 1)
*/

void lcdClearRow( byte rowID) {
  // Move to beginning of the row
  lcd.setCursor(0,rowID);
  
  // Fill with spaces
  for (int i =0 ; i < LCDCOLS; i++) {lcd.print(" "); }  
  
  // move again to beginning
  lcd.setCursor (0,rowID);
}



/*----------------------------------------------------------------------------------------------------------------------
Sub: lcdPrintAction 
-------------------------------
This sub displays the results of the action performed (SAVE/CLEAR). 
Message is written in the left bottom part of LCD
 
Invoked by:
 . configBaudRate
 . configRtxAddress
 . configIC735Mode
 . configBandToAntenna
 . configAntennaName
 . configFactoryDefaults

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: 
 . ACTIONDELAY
 
Arguments: 
 . action: 0 --> Cancel    1 --> Save
*/

void lcdPrintAction (boolean action) {
  // Remove LCD blinking
  lcd.noBlink();
  
  // Move cursor to proper position
  lcd.setCursor(6,1);

  // Print the value
  if (action) {lcd.print("   * SAVED");} else {lcd.print("  * CANCEL");}

  // Wait 
  delay(ACTIONDELAY);
}

