/* 
   --------------------------------------------------------------------------------------- 
   File        : CI_V_based_Remote_Antenna_Switch.pde
   Author      : IK5PWC, Civinini Luca - luca@ik5pwc.it - http://www.ik5pwc.it
   Created     : 2011/09/20 
   Description : This is the main file. Purpose of his sketch/project is to remotely
                 control an antenna switch based on data coming from ICOM Cat interface.
   
   Licensed freely only to authorized amateur radio stations. Any other use is forbidden.
   Use this code at your own risk.
   Code can be modified and redistributed under following  conditions:
   1) You should not remove any reference to original author
   2) Your changes must be forwared to original  author
   3) Any commercial activity is forbidden
   
   CI-V and Icom are registered trademarks
   --------------------------------------------------------------------------------------- 
*/



/* Preprocessor Macro */
#define ANTENNANAMELEN 9             // Length for antenna names
#define NUMBANDS 11                  // Number of bands available
#define NUMANTENNA 7                 // switch positions
#define VERSION "1.4"                // Program version


/* ----------------------------------- GLOBAL CONSTANST DATA ------------------------------- */

/*                           bandValue      0    1    2    3     4     5     6     7     8     9    10    11  */         // 0 is invalid */
const byte        bandMeters[NUMBANDS+1] = {0, 160,  80,  40,   30,   20,   17,   15,   13,   10,    6,    4};           // band (in meters)
const long           bandLow[NUMBANDS+1] = {0,1800,3500,7000,10100,14000,18068,21000,24890,28000,50000,70100};           // lower frequency limit for each HF band
const long          bandHigh[NUMBANDS+1] = {0,1900,3800,7200,10150,14350,18168,21450,24990,29700,51000,70300};           // Upper frequency limit for each HF band
const int                    baudRates[] = {0,300,1200,4800,9600,19200};                                                 // Available baud rates for CI-V protocol

const char validChars[] ={" ABCDEFGHIJKLMNOPQRSTUVWXYZ abcdefghijklmnopqrstuvwxyz 1234567890 +-*/)(&%.,<>^#@"};   // list of valid chars to be used within antenna names




/* ------------------------------- GLOBAL VARIABLES DECLATATION ---------------------------- */

byte          rtxAddr = 0 ;                 // RTX address
boolean     ic735mode = false;              // used to check for 4 or 5 byte frequency format
byte    baudRateIndex = 0;                  // Serial speed index (checked on baudRates array)
boolean notConfigured = true;               // there is a valid configuration in EEPROM? By default no

char antennaName[(NUMANTENNA+1)*ANTENNANAMELEN];            // Stores names for antennas
byte  bandToAnt[NUMBANDS+1];                                // Band to antenna association




/*
----------------------------------------------------------------------------------------------------------------------
Sub: setup
-------------------------------
This sub read configuration from EEPROM, start the serial interface and plot initial info on 
display

Invoked by:
 . CPU Startup

Called Sub/Functions:  
 . readConfig()
 . lcdInitWithBanner()
 . lcdUpdate()
 . initDefaults()
 . initOutput()
 . loop() (implicit)

Global Const Used: 
 . baudRates[]

Global variables used:
 . notConfigured
 . baudRateIndex

Pre Processor Macro: NONE
 
Arguments: NONE
*/

void setup() {                
  // Read configuration from EEPROM
  readConfig();
  
  // If it's the first power on, then load default values
  if (notConfigured) {initDefaults();}

  // Init serial ports
  Serial.begin (baudRates[baudRateIndex]);

  // Init display
  lcdInitWithBanner();
  lcdUpdate(0,0,0,true);
   
  // Init output pins
  initOutput();
}



/*
----------------------------------------------------------------------------------------------------------------------
Sub: loop
-------------------------------
Main program body: it reads data from CIV, check key status and switch the proper antenna or enter the menu

Invoked by:
 . setup

Called Sub/Functions:  
 . readCIVData()
 . decodeBand()
 . readKey()
 . configMainMenu()
 . lcdUpdate()
 . displayMessage()
 . switchAntenna()
 . checkErrors()
 . warningLed()

Global Const Used: NONE

Global variables used: NONE

Pre Processor Macro: NONE

Arguments: NONE
*/

void loop() {
  static long         curFreq      = 0;           // Current  frequency value
  static byte         curBand      = 0;           // Current band value
  static byte         curAntenna   = 0;           // Current selected antenna
  static boolean      autoMode     = true;        // By default I start in auto mode
  static byte         deviceStatus = 0;           // Report current operation status
  int                 switchOpCode = 0;           // switching operation code 
                                                  //    -1 decrement antenna
                                                  //    +1 increment 
                                                  //    0xFF00 do nothing
                                                  //    0xFF01 switch based on band
                   
  // Get CI-V data from the serial port. In case there is no data, then return the current freq value
  curFreq = readCIVData(curFreq); 

  // decode the band
  curBand = decodeBand(curFreq);

  // Do an action depending on button status
  switch (readKey()) {
     case 1:        // UP
        // Set in manual mode and set opcode to next ant
        autoMode = false;
        switchOpCode = 1;
        break;
        
     case 2:        // DOWN
        // Set in manual mode and set opcode to next ant
        autoMode = false;
        switchOpCode = -1;
        break;

     case 4:        // ESC
         // if we were in manual, go back to auto and switch
        autoMode = true;
        switchOpCode = 0xFF01;
        break;
     
     case 8:        // MENU
        configMainMenu();
        lcdUpdate(curFreq,curBand,curAntenna,true);
        break;    

     case 16:       // MESSAGE
        if (deviceStatus != 0x00) {
            displayMessage(deviceStatus);
            lcdUpdate(curFreq,curBand,curAntenna,true);
        }
        break;    

     default:
        // if no key has been pressed, then the operation should be based on previous state
        // whenever it's manual or auto.
        switchOpCode = 0xFF00 | autoMode;
        break;
  }  
  
  // Switch the antenna based on band only if in manual mode
  curAntenna = switchAntenna(curBand, switchOpCode, curAntenna);  

  // Update display
  lcdUpdate(curFreq, curBand, curAntenna, false);
  
  // Check for warnings
  deviceStatus =  checkErrors(curBand, curAntenna, curFreq, autoMode);
  warningLed (deviceStatus);
}

