/* 
   --------------------------------------------------------------------------------------- 
   File        : C_EEPROM.pde
   Author      : IK5PWC, Civinini Luca - luca@ik5pwc.it - http://www.ik5pwc.it
   Created     : 2011/09/20 
   Description : This file contais all functions for EEPROM Management
   
   Licensed freely only to authorized amateur radio stations. Any other use is forbidden.
   Use this code at your own risk.
   Code can be modified and redistributed under following  conditions:
   1) You should not remove any reference to original author
   2) Your changes must be forwared to original  author
   3) Any commercial activity is forbidden
   
   CI-V and Icom are registered trademarks
   --------------------------------------------------------------------------------------- 
*/

/* 0x0000 --> 0x03FF
   EEPROM Address       VARIABLE
   ----------------------------------------------------
   0x0000               ** NOT USED
   0x0001               notConfigured   
   0x0002               rtxAddr
   0x0003               ic735mode
   0x0004               baudRateIndex
   0x0005               ** NOT USED
   0x0006 - 0x000F      ** RESERVED FOR FUTURE USE
   0x0010               bandToAnt[0]
   0x0011               bandToAnt[1]
   0x0012               bandToAnt[2]
   0x0013               bandToAnt[3]
   0x0014               bandToAnt[4]
   0x0015               bandToAnt[5]
   0x0016               bandToAnt[6]
   0x0017               bandToAnt[7]
   0x0018               bandToAnt[8]
   0x0019               bandToAnt[9]
   0x001A               bandToAnt[10]
   0x001B - 0x00FF      ** RESERVED FOR FUTURE USE
   0x0100 - 0x0108      Name Antenna 0
   0x0110 - 0x0118      Name Antenna 1
   0x0120 - 0x0128      Name Antenna 2
   0x0130 - 0x0138      Name Antenna 3
   0x0140 - 0x0148      Name Antenna 4
   0x0150 - 0x0158      Name Antenna 5
   0x0160 - 0x0168      Name Antenna 6
   0x0170 - 0x0178      Name Antenna 7  
   0x0179 - 0x03FF      ** RESERVED FOR FUTURE USE
*/

// Include EEPROM library
#include <EEPROM.h>

/*
----------------------------------------------------------------------------------------------------------------------
Sub: readConfig
-------------------------------
This function read config from EEPROM

Invoked by:
 . setup

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used: 
 . rtxAddr
 . ic735mode
 . baudRateIndex
 . bandToAnt[]
 . antennaName[]
 

Pre Processor Macro: 
 . NUMANTENNA
 
Arguments: NONE
*/

void readConfig() {
   
   // Read
   notConfigured  = EEPROM.read(0x0001);
   rtxAddr        = EEPROM.read(0x0002);
   ic735mode      = EEPROM.read(0x0003);
   baudRateIndex  = EEPROM.read(0x0004);

   // Band to antenna config
   for (int i=0; i<=NUMBANDS; i++) {
       bandToAnt[i] = EEPROM.read(0x0010+i);
   }
   
   // Antenna Names
   for (int i=0; i<(NUMANTENNA+1)*ANTENNANAMELEN;i++) { 
     antennaName[i] = char(EEPROM.read(0x0100+i));
   }  
}



/*
----------------------------------------------------------------------------------------------------------------------
Sub: saveConfig
-------------------------------
This function write config from EEPROM
 

Invoked by:
 . configMainMenu

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used: 
 . rtxAddr
 . ic735mode
 . baudRateIndex
 . bandToAnt[]
 . antennaName[]
 
Pre Processor Macro: 
 . NUMANTENNA
 
Arguments: NONE
*/

void saveConfig() {

   // Save general data
   EEPROM.write(0x0001,0x00);
   EEPROM.write(0x0002,rtxAddr);
   EEPROM.write(0x0003, ic735mode);
   EEPROM.write(0x0004,baudRateIndex);
   
   // Band to antenna config
   for (int i=0; i<=NUMBANDS; i++) {
       EEPROM.write(0x0010+i,bandToAnt[i]);
   }
   
   // Antenna Names
   for (int i=0; i<(NUMANTENNA+1)*ANTENNANAMELEN;i++) { 
       EEPROM.write(0x0100+i,byte(antennaName[i]));
   }
}

