/* 
   --------------------------------------------------------------------------------------- 
   File        : D_CIV.pde
   Author      : IK5PWC, Civinini Luca - luca@ik5pwc.it - http://www.ik5pwc.it
   Created     : 2011/09/20 
   Description : This file contais all functions related to CIV communication
   
   Licensed freely only to authorized amateur radio stations. Any other use is forbidden.
   Use this code at your own risk.
   Code can be modified and redistributed under following  conditions:
   1) You should not remove any reference to original author
   2) Your changes must be forwared to original  author
   3) Any commercial activity is forbidden
   
   CI-V and Icom are registered trademarks
   --------------------------------------------------------------------------------------- 
*/

/* Preprocessor Macro */
#define CIVFREQLEN 12                // Size of sliding window to get CIV Data



/*----------------------------------------------------------------------------------------------------------------------
Function: readCIVData
-------------------------------
 This function read data from serial port looking
 for a valid frequency command. It uses the freqWin
 static variable as a sliding window area to store 
 data received from serial. The function needs
 the default value to be returned in case no
 requency data is available (civFreq)
 
Invoked by:
 . loop

Called Sub/Functions: 
 . isFreqData
 . decodeFrequency
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: 
 . CIVFREQLEN

Arguments: 
 . civFreq: current frequency
*/

long readCIVData(long civFreq) {
  static byte freqWin[CIVFREQLEN];  /* This array is a fixed size sliding window for looking icoming serial
                                       stream and find a frequency data. Its size is fixed to 12 chars */

  byte serBuffer = 0;               // Number of chars in the serial buffer (buffer is 128 bytes)

  // Get the current Serial buffer size
  serBuffer = Serial.available();

  // Read data from buffer, if any
  for (int i=0; i < serBuffer ;  i++) {

    // Shift all positions in the frequency sliding windos
    for (int j=0; j < CIVFREQLEN - 2 ; j++) { freqWin[j] = freqWin[j+1] ; }

    // Append one char to the end of the window
    freqWin[CIVFREQLEN-2] = Serial.read();

    // Now it's possbile that the sliding freq window contains a valid frequency data and, in case, return its value
    if (isFreqData(freqWin)) {civFreq = decodeFrequency (freqWin,civFreq);}
  }
  
  // Return the value
  return (civFreq);
}  



/*----------------------------------------------------------------------------------------------------------------------
Function: isFreqData
-------------------------------
 This function returns true if the passed array window
 contains freq data, otherwise return false.
 
Invoked by:
 . readCIVData

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used: 
 . ic735mode
 . rtxAddr
 
Pre Processor Macro: NONE

Arguments: 
 . freqCIVData: an array containing the sliding window used during serial rx
*/

boolean isFreqData (byte freqCIVData[]) {
  int i;      // index 

  // If 735 mode is enabled, then I have to start from index 1 otherwise from index 0
  (ic735mode) && (i=1) || (i=0);

  if ( freqCIVData[i]    == 0xFE && freqCIVData[i+1]  == 0xFE &&               // Begin Frame
       freqCIVData[i+3]  == rtxAddr &&                                         // From Address = RTX Address
       (freqCIVData[i+4] == 0x03 || freqCIVData[i+4]  == 0x00) &&              // Read or set freq
       freqCIVData[10]   == 0xFD  )                                            // End of Frame
  { return(true); } else {return(false);}    
}




/*----------------------------------------------------------------------------------------------------------------------
Function: decodeFrequency
-------------------------------
 This function returns a decoded frequency to the caller.
 It needs an array containing received data and a default
 frequency value to return in case there is no valid data
 within the array.
 WEhen decoding, it takes only digits from 1 KHz to 10 MHz
 (for HF + 50 MHz band they are enought)
 
Invoked by:
 . readCIVData

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used: 
 . ic735mode
 
Pre Processor Macro: NONE 

Arguments: 
 . freqCIVData[]: an array containing the sliding window used during serial rx
 . defFreq: contains the current frequency
*/

long decodeFrequency (byte freqCIVData[], long defFreq) {
  unsigned long dataFreq = defFreq;  // Temp variable  to store the frequency 
  int i;                             // index 

  // If 735 mode is enabled, then I have to start from index 1 otherwise from index 0
  (ic735mode) && (i=1) || (i=0);

              /*  4 bit LSB */                      /* 4 bit MSB*/
  dataFreq =                                        long((freqCIVData[i+6] >> 4)*1E3);                      // 100Hz - 1 KHz
  dataFreq += long((freqCIVData[i+7] & 0x0F)*1E4) + long((freqCIVData[i+7] >> 4)*1E5);                      // 10 KHz - 100 KHz
  dataFreq += long((freqCIVData[i+8] & 0x0F)*1E6) + long((freqCIVData[i+8] >> 4)*1E7);                      // 1 MHz - 10 MHz
  if (!ic735mode) {dataFreq += long((freqCIVData[i+9] & 0x0F)*1E8) + long((freqCIVData[i+9] >> 4)*1E9);}    // 100 MHz - 1 GHz (for non IC735)

  // Scale the value in KHz
  dataFreq /= 1E3;

  // if the value is larger than 65536 then return 0 otherwise return the value
  if (dataFreq > 80000) {return (0);} else {  return (dataFreq);}
}

