/* 
   --------------------------------------------------------------------------------------- 
   File        : E_io.pde
   Author      : IK5PWC, Civinini Luca - luca@ik5pwc.it - http://www.ik5pwc.it
   Created     : 2011/09/20 
   Description : This file contais all I/O functions (key and out to antenna switch)
   
   Licensed freely only to authorized amateur radio stations. Any other use is forbidden.
   Use this code at your own risk.
   Code can be modified and redistributed under following  conditions:
   1) You should not remove any reference to original author
   2) Your changes must be forwared to original  author
   3) Any commercial activity is forbidden
   
   CI-V and Icom are registered trademarks
   
   --------------------------------------------------------------------------------------- 
*/

/* Preprocessor Macro */
#define KEYPIN1 A0                   // Pin used for connecting the keypad
#define KEYPIN2 A1                   // Pin used for connecting the message button
#define XMITPIN A2                   // Pin use to monitor if Txing
#define DEBOUNCETIME 20              // milliseconds to wait between two different read for input buttons         
#define OUT1 3                       // Output pins for BCD out - LSB
#define OUT2 4                       // Output pins for BCD out 
#define OUT3 5                       // Output pins for BCD out - MSB
#define WARNING 7                    // Warning led
#define BLINKDURATION 700           // Blinking duration in millisec
#define STARTREPEAT  1.1             // seconds after starting repeating keypad
#define REPEATFREQ 100               // milliseconds between repeating



/*----------------------------------------------------------------------------------------------------------------------
Function: readKey
-------------------------------
 This function read button status. Buttons are
 connected to analog line using a resistor network
 to identify each button. Standard is:
  0,00Vref --> UP
  0,25Vref --> DOWN
  0,50Vref --> ESC
  0,75Vref --> MENU
  
  ADC is 10 bit and voltage is  checked two times
  to avoid bounces.
 
 This function returns a byte representing the presed key
 only if the status is different from the previous call
 (i.e. the key must be released and pressed again to 
 get an o
  UP   --> 0x01
  DOWN --> 0x02 
  ESC  --> 0x04
  MENU --> 0x08
  MESG --> 0x10 
 
Invoked by:
 . loop
 . configMainMenu
 . configBaudRate
 . configRtxAddress
 . configIC735Mode
 . configBandToAntenna
 . configAntennaName
 . configFactoryDefaults
 . configSoftwareVersion
 . displayMessage

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: 
 . KEYPIN1
 . KEYPIN2
 . DEBOUNCETIME
 . REPEATFREQ
 . STARTREPEAT

Arguments: NONE
*/

byte readKey() {
    int keyValue1 =0 ;                // Store value read from analog input - first pass
    int keyValue2 =0 ;                // Store value read from analog input - second pass
    static byte oldKeyStatus = 0;     // Prevoius keyboard state
    byte tmpKey = 0 ;                 // Temp key value

    static boolean repeating = false;
    static unsigned long keyLastStatusTimestamp = 0;

    // Read analog input, wait 100 millisec and read it again
    // to avoid bounces.
    keyValue1 = analogRead(KEYPIN1); delay (DEBOUNCETIME); keyValue2 = analogRead(KEYPIN1);

    // if values have the 3 MSB bit different, then there was an error, so exit reporting 0
    if ((keyValue1 & 0x03F0) != (keyValue2 & 0x03F0)) { return(tmpKey);}

    // get the value corresponding to the pressed key. Simply divide the binary value (0 --> 1023)
    // for a proper value and filter record with quotient 1.
    if (keyValue1 <  100) {tmpKey=1;}
    if ((keyValue1 / 200) == 1 ) {tmpKey=2;}
    if ((keyValue1 / 470) == 1 ) {tmpKey=8;}
    if ((keyValue1 / 620) == 1 ) {tmpKey=4;}
    if (keyValue1 > 950)  { 
        // Key1 is not pressed, so start reading KEY2
        // First of all report the value to 0 (should be 8 because input is pulled-up so in case of no input this should be the last value checked)
        tmpKey = 0 ;

        // Read analog input, wait 100 millisec and read it again to avoid bounces.
        keyValue1 = analogRead(KEYPIN2); delay (DEBOUNCETIME); keyValue2 = analogRead(KEYPIN2);
  
        // if values have the 3 MSB bit different, then there was an error, so exit reporting 0
        if ((keyValue1 & 0x03F0) != (keyValue2 & 0x03F0)) { return(tmpKey);}

        // Report if down        
        if (keyValue1 <  100) {tmpKey=16;}  else {tmpKey=0;} 
    }        
   
    // Check if new and old value are different
    if (oldKeyStatus != tmpKey) {
        // Yes, so key has been pressed and released (or different key has been pushed). Report the new value, set the timestamp and disable repeating
        oldKeyStatus = tmpKey;
        keyLastStatusTimestamp = millis();
        repeating = false;
    } else {        
        // New and old are the same,
        // If repeating is enabled and key has been pressed longer than REPEATFREQ, report the current value updating timestamp
        if ((repeating) && (millis() - keyLastStatusTimestamp > REPEATFREQ)) {
            // Ok, start repeating...
            oldKeyStatus = tmpKey;
            keyLastStatusTimestamp = millis();
         } else {
            // No, repeating is not enabled, key is the same that last function call, so report nothing
            tmpKey = 0;
         }
       
         //Check if the key has been pressed for longer than STARTREPEAT frequency to enable repeating
         if ((millis() - keyLastStatusTimestamp > STARTREPEAT*1E3 ) && (keyLastStatusTimestamp>0)) {repeating = true; }
    }
    
    // Return the tmpkey value
    return (tmpKey);
}



/*----------------------------------------------------------------------------------------------------------------------
Function: switchAntenna
-------------------------------
 This function is responsible to send to output pins
 the proper value for switching the remote unit.
 The function returns the switched antenna. 
 
Invoked by:
 . loop

Called Sub/Functions: NONE
                
Global Const Used: 
 . bandToAnt[]

Global variables used: NONE
 
Pre Processor Macro: 
 . XMITPIN
 . DEBOUNCE
 . NUMANTENNA
 . OUT1
 . OUT2
 . OUT3

Arguments: 
 . bandIndex: is the current band to check for (if 0 no check is done)
 . upOrDown:  this a multi-valued field.
               -1       switch to previous antenna
               +1       switch to next antenna
               0xFF01   Automatic mode enabled, check the bandIndex
               0xFF00   Manual mode enabled but no switching is required
 . selAntenna: the currently selected antenna
*/

byte switchAntenna (byte bandIndex, int upOrDown, byte selAntenna) {
    int txValue1 =0 ;                // Store value read from analog input for txing
    int txValue2 =0 ;                // Store value read from analog input for txing

    // Read analog input for tx status, wait 100 millisec and read it again
    // to avoid bounces.
    txValue1 = analogRead(XMITPIN); delay (DEBOUNCETIME); txValue2 = analogRead(XMITPIN);

    // if values have the 3 MSB bit different, then there was an error, so exit reporting 0
    if ((txValue1 & 0x03F0) != (txValue2 & 0x03F0)) { return(selAntenna);}
  
    // if transmitting, then immediatly return the selected antenna
    if (txValue1 <  600) { return(selAntenna); }

  // In automatic mode, check the antenna value for a valid band
  if ((upOrDown == 0xFF01) && (bandIndex != 0)  && (bandToAnt[bandIndex] > 0)) {
    // OK, an antenna is defined

    // Check if it's enabled and save the value in selAntenna
    selAntenna = bandToAnt[bandIndex];
  } 

  // We are in manual mode if the value in upOrDown is not FF01
  if ((upOrDown == -1) || (upOrDown == 1))  {
    // in this case bandValue contains -1 or +1 (depending which button has been pressed)
    
    // move the antenna index up or down
    selAntenna +=  upOrDown;
            
    // Check if we are at the end or not
    switch (selAntenna) {
      case 0:
          selAntenna = NUMANTENNA;
          break;                     
      case 255:         
          selAntenna = NUMANTENNA;
          break;                     
      case NUMANTENNA+1:
          selAntenna = 1;
          break;
    }                  
  }
  
  // Swith the antenna sending the corresponding HEX code to output pins
  digitalWrite(OUT1,byte(selAntenna) & 0x01);
  digitalWrite(OUT2,byte(selAntenna) & 0x02);
  digitalWrite(OUT3,byte(selAntenna) & 0x04);

  // Return currently selected antenna
  return(selAntenna);
}



/*
----------------------------------------------------------------------------------------------------------------------
Sub: warningLed
-------------------------------
This sub check for errors and warnings 

Invoked by:
 . loop

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used:  NONE

Pre Processor Macro: 
 . WARNING
 . BLINKDURATION
 
Arguments: 
 . message: byte indicating if warning led is off (0x00), blinks (0x08) or on (!= 0x00)
*/

void warningLed(byte message) {
   static unsigned long startBlinking = 0;
   static boolean blinkStatus = false;
   
   if ((message != 0x0) && (message != 0x8)) {digitalWrite(WARNING,HIGH);} else {digitalWrite(WARNING,LOW);}  
   
   // If in manual mode then blink
   if (message == 0x8) {
      if (millis()-startBlinking > BLINKDURATION) {
         startBlinking = millis();
         blinkStatus = ! blinkStatus;
      }

      digitalWrite(WARNING,blinkStatus);
   }  
}



/*----------------------------------------------------------------------------------------------------------------------
Sub: initOutput
-------------------------------
 This sub initializes the output pins
 
Invoked by:
 . setup

Called Sub/Functions: NONE
                
Global Const Used: None

Global variables used: NONE
 
Pre Processor Macro: 
 . OUT1
 . OUT2
 . OUT3
 . WARNING
 
Arguments: NONE
*/

void initOutput() {
  // Switch output
  pinMode (OUT1, OUTPUT);
  pinMode (OUT2, OUTPUT);
  pinMode (OUT3, OUTPUT);

  
  // Warning Led
  pinMode (WARNING,OUTPUT);
}
