/* 
   --------------------------------------------------------------------------------------- 
   File        : F_miscFunctions.pde
   Author      : IK5PWC, Civinini Luca - luca@ik5pwc.it - http://www.ik5pwc.it
   Created     : 2011/09/20 
   Description : This file misc functions used within the project
   
   Licensed freely only to authorized amateur radio stations. Any other use is forbidden.
   Use this code at your own risk.
   Code can be modified and redistributed under following  conditions:
   1) You should not remove any reference to original author
   2) Your changes must be forwared to original  author
   3) Any commercial activity is forbidden
   
   CI-V and Icom are registered trademarks
   --------------------------------------------------------------------------------------- 
*/

#define CIVNODATA 3600            // Seconds before triggering no data from radio

/*----------------------------------------------------------------------------------------------------------------------
Function: decodeBand
-------------------------------
 This function returns an integer rapresenting an index  to find the 
 corresponding band name in the proper array.  If the frequency doesn't 
 belong to any defined band, then  it returns 0.
 
Invoked by:
 . loop

Called Sub/Functions: NONE
                
Global Const Used: 
 . bandLow[]
 . bandHigh[]

Global variables used: NONE
 
Pre Processor Macro: 
 . NUMBANDS
 
Arguments: 
 . freqItem: an unsigned in representing the frequency in KHz
*/

byte decodeBand (long freqItem) {
  int  divResult =  0 ;          // result of division
  byte bandInteger = 0;         // function return value (default is invalid)
  
  // Cycle all bandLow items. 
  for (int i = 1; i <= NUMBANDS ; i++) {
    // Get the quotient
    divResult = freqItem / bandLow[i];
    
    // The proper band is the one wher previous division is 1 and
    // the frequency is under the upper band limit, then return an
    // integer which represent the band and exit for
    if ((divResult == 1) && (freqItem <= bandHigh[i])) { 
      bandInteger=i;
      break;
    }
  } 
  
  // Return the value
  return (bandInteger);
}


/*----------------------------------------------------------------------------------------------------------------------
Sub: initDefaults
-------------------------------
This sub resets the device to default values
 
Invoked by:
 . setup
 . configFactoryDefaults

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used: 
 . rtxAddr
 . ic735mode
 . baudRateIndex
 . notConfigured
 . bandToAnt[]
 . antennaName[]
 
Pre Processor Macro: 
 . NUMBANDS
 . ANTENNANAMELEN
 
Arguments: NONE
*/

void initDefaults() {

  rtxAddr = 0x2C ;              // Default RTX Address
  ic735mode = false;            // Default no IC735
  baudRateIndex = 4;            // Default 9600
  notConfigured = false;        // Now is configured (altought with defaults!)

  // By default no antenna in any band
  for (int i =0 ; i<=NUMBANDS;i++) {bandToAnt[i]=0;}

  // first antenna position has a fixed name (NoAntenna)
  antennaName[0] = 'N';
  antennaName[1] = 'o';
  antennaName[2] = 'A';
  antennaName[3] = 'n';
  antennaName[4] = 't';
  antennaName[5] = 'e';
  antennaName[6] = 'n';
  antennaName[7] = 'n';
  antennaName[8] = 'a';

  // Other antennas get the name Antenna 1, Antenna 2 ........
  for (int i =1; i<= NUMANTENNA;i++) {
    antennaName[i*ANTENNANAMELEN+0] = 'A';
    antennaName[i*ANTENNANAMELEN+1] = 'n';
    antennaName[i*ANTENNANAMELEN+2] = 't';
    antennaName[i*ANTENNANAMELEN+3] = 'e';
    antennaName[i*ANTENNANAMELEN+4] = 'n';
    antennaName[i*ANTENNANAMELEN+5] = 'n';
    antennaName[i*ANTENNANAMELEN+6] = 'a';
    antennaName[i*ANTENNANAMELEN+7] = ' ';
    antennaName[i*ANTENNANAMELEN+8] = 48+i;    // I should store the ASCII value for i (integer starts from 48)
  }
}



/*----------------------------------------------------------------------------------------------------------------------
Function: checkErrors
-------------------------------
This function return an error code for display
 0x00 --> All OK
 0x01 --> Out of Band
 0x02 --> No antenna for band
 0x04 --> No CIV data
 0x08 --> Manual mode enabled
 0x10 --> No antenna connected
 
Invoked by:
 . loop

Called Sub/Functions: NONE
                
Global Const Used: NONE

Global variables used: NONE
 
Pre Processor Macro: 
 . CIVNODATA
 
Arguments: 
 . bandIndex: the current selected band as an index to band array
 . antenna: the currently selected antenna
 . curFreq: an integer for current freq in KHz
 . automatic: true if in auto mode, false otherwise
*/

byte checkErrors(byte bandIndex, byte antenna, unsigned int curFreq, boolean automatic) {
  byte tmpStatus = 0;                      // Temp var for reporting status
  static unsigned long timestamp = 0;      // Timestamp for valid frequency data
  static unsigned int oldFreq = 0;         // Store frequency
  
  // Update the timestamp
  if (curFreq != oldFreq) {oldFreq = curFreq; timestamp = millis();}

  if (bandIndex == 0) {tmpStatus= 0x01;}                                  // Out of band
  if ((bandToAnt[bandIndex] == 0) && (bandIndex !=0)) {tmpStatus= 0x02;}  // No antenna for band
  if (millis()-timestamp > CIVNODATA*1E3) {tmpStatus= 0x04;}              // No data from radio
  if (!automatic) {tmpStatus= 0x08;}                                      // Manual Mode
  if (antenna == 0) {tmpStatus= 0x10;}                                    // No antenna connected
 
  // return the value
  return(tmpStatus);
}

